<?php

/**
 * @file
 * Handles drush command gravect.
 */

use Symfony\Component\Yaml\Yaml;


/**
 * Drush command logic to export content types.
 */
function drush_grav_export_content_types() {
  drush_print("Beginning content type export.");
  $templates_export_folder = "public://grav_export/user-" . date('Ymd') . "/themes/drupal_export/templates/";
  $blueprints_export_folder = "public://grav_export/user-" . date('Ymd') . "/themes/drupal_export/blueprints/";
  file_prepare_directory($templates_export_folder, FILE_CREATE_DIRECTORY);
  file_prepare_directory($blueprints_export_folder, FILE_CREATE_DIRECTORY);

  $content_types = node_type_get_types();
  foreach ($content_types as $content_type) {

    $template_content = "{% extends 'partials/base.html.twig' %}\n\n{% block content %}\n    {{ page.content }}\n{% endblock %}\n";
    file_save_data($template_content, $templates_export_folder . $content_type->type . ".html.twig", TRUE);

    $blueprint_component = drupal_get_path('module', 'grav_export') . "/grav_components/contentType_blueprint.yaml";
    $blueprint = Yaml::parseFile($blueprint_component);
    $blueprint['title'] = $content_type->name;

    // Reset $new_fields for each content type.
    $new_fields = NULL;

    $fields = field_info_instances('node', $content_type->type);
    foreach ($fields as $field_key => $field) {
      // todo: identify other fields and add below.
      $field_name = $field['field_name'];
      $field_info = field_info_field_by_id($field['field_id']);

      switch ($field_info['type']) {
        case "addressfield":
          $new_fields["header." . $field_name] = array(
            "label" => $field['label'],
            "type" => "array",
            "help" => strip_tags($field['description']),
            "placeholder_key" => 'PLUGIN_ADMIN.METADATA_KEY',
            "placeholder_value" => 'PLUGIN_ADMIN.METADATA_VALUE',
          );
          break;

        case "datetime":
          $new_fields["header." . $field_name] = array(
            "label" => $field['label'],
            "type" => "datetime",
            "help" => strip_tags($field['description']),
          );
          break;

        case "email":
          $new_fields["header." . $field_name] = array(
            "label" => $field['label'],
            "type" => "email",
            "help" => strip_tags($field['description']),
          );
          break;

        case "file":
          $new_fields["header." . $field_name] = array(
            "label" => $field['label'],
            "type" => "file",
            "help" => strip_tags($field['description']) . " | Available file types: " . $field['settings']['file_extensions'],
            "destination" => "user/data/files/" . preg_replace('/\_/', '/', $field_name) . '/',
          );
          if ($field['settings']['file_directory']) {
            $new_fields["header." . $field_name]['destination'] = "user/data/files/" . $field['settings']['file_directory'];
          }
          $extensions = explode(" ", $field['settings']['file_extensions']);

          foreach ($extensions as $extension) {
            $new_fields["header." . $field_name]["accept"][] = $extension;
          }

          if ($field_info['cardinality'] != 1) {
            $new_fields["header." . $field_name]['multiple'] = TRUE;
          }
          else {
            $new_fields["header." . $field_name]['multiple'] = FALSE;
          }
          break;

        case "image":
          $new_fields["header." . $field_name] = array(
            "label" => $field['label'],
            "type" => "file",
            "help" => strip_tags($field['description']) . " | Available file types: " . $field['settings']['file_extensions'],
            "destination" => "user/data/files/" . preg_replace('/\_/', '/', $field_name) . '/',
            "accept" => array('image/*'),
          );
          if ($field_info['cardinality'] != 1) {
            $new_fields["header." . $field_name]['multiple'] = TRUE;
          }
          else {
            $new_fields["header." . $field_name]['multiple'] = FALSE;
          }
          $extensions = explode(" ", $field['settings']['file_extensions']);
          foreach ($extensions as $extension) {
            $new_fields["header." . $field_name]["accept"][] = $extension;
          }
          break;

        case "link_field":
          $new_fields["header." . $field_name] = array(
            "help" => $field['description'],
            "label" => $field['label'],
            "type" => "text",
          );
          break;

        case "list_boolean":
          $new_fields["header." . $field_name] = array(
            "help" => $field['description'],
            "label" => $field['label'],
            "type" => "toggle",
            "options" => array(
              "1" => "True",
              "0" => "False",
            ),
            "highlight" => "1",
            "validate" => array(
              "type" => "bool",
            ),
          );
          break;

        case "number_decimal":
          $new_fields["header." . $field_name] = array(
            "help" => $field['description'],
            "label" => $field['label'],
            "type" => "text",
            "validate" => [
              "type" => "number"
            ],
          );
        break;

        case "number_integer":
          $new_fields["header." . $field_name] = array(
            "help" => $field['description'],
            "label" => $field['label'],
            "type" => "text",
            "validate" => [
              "type" => int
            ],
          );
        break;

        case "taxonomy_term_reference":
          // Nothing needs to be done for taxonomy terms as Grav includes
          // this on the Options tab.
          break;

        case "text":
          $new_fields["header." . $field_name] = array(
            "help" => $field['description'],
            "label" => $field['label'],
            "type" => "text",
          );
          break;

        case "text_long":
          $new_fields["header." . $field_name] = array(
            "help" => $field['description'],
            "label" => $field['label'],
            "type" => "textarea",
            "rows" => $field['widget']['settings']['rows'],
          );
          break;

        case "text_with_summary":
          // Drupal body field belongs in frontmatter.  Don't create a field for it.
          if ($field_name == 'body') {
            break;
          }
          // todo: Finish text_with_summary export.
          break;

        default:
          drush_print("\nMissing field definition: '" . $field_info['type'] . "' (" . $field['field_name'] . ") in '$content_type->name'.");
          $missing = TRUE;
      }
    }

    if ($new_fields) {
      $blueprint['form']['fields']['tabs']['fields']['content']['fields'] = $new_fields;
    }
    else {
      unset($blueprint['form']);
    }

    $yaml_output = Yaml::dump($blueprint, 20, 4);
    file_save_data($yaml_output, $blueprints_export_folder . $content_type->type . ".yaml", TRUE);
  }

  drush_print("\n\nSave Complete!  " . count($content_types) . " content types exported.");
  drush_print("Templates data is located in $templates_export_folder");
  drush_print("Blueprints data is located in $blueprints_export_folder");
  if ($missing) {
    drush_print("\nSome content may be missing in the exported page due to missing field definitions.\n");
  }
}
